/** Simple program for calculating the smallest 
 * surrounding rectangle of two rectangles.
 *
 * @author robert.binna@uibk.ac.at
 * @date   09.04.2008
 */

#include <math.h>
#include <stdio.h>

struct point {
	int x;
	int y;
};

struct rectangle {
	struct point a;
	struct point b;
};

/**
 * Calculates the maximum value of two integer values.
 * If both are equal the second value will be chosen, 
 * which is in this case the same as the first.
 *
 * @param first the first value
 * @param second the second value
 * @return the maximum value
 */
int max(int first, int second) {
	return (first > second) ? first : second;
}

/**
 * Calculates the minimum value of two integer values
 * If both are equal the second value will be chosen, 
 * which is in this case the same as the first.
 *
 * @param first the first value
 * @param second the second value
 * @return the minimum value
 */
int min(int first, int second) {
	return (first < second) ? first : second;
}

/**
 * Function that calculates the smallest surrounding rectangle of two 
 * arbitrary rectangles.
 * Calculates the minimum and maximum x/y values, and creates
 * a new rectangle of this coordinates.
 *
 *
 * @param rect1 the first rectangle 
 * @param rect2 the second rectangle
 * @return the smallest surrounding rectangle
 */
struct rectangle smallestSurroundingRect(
	struct rectangle rect1, struct rectangle rect2
) {
	int maxX = max(
		max(rect1.a.x,rect2.a.x)
		, max(rect1.b.x,rect2.b.x)
	);
	int maxY = max(
		max(rect1.a.y,rect2.a.y)
		, max(rect1.b.y,rect2.b.y)
	);
	
	int minX = min(
		min(rect1.a.x,rect2.a.x)
		, min(rect1.b.x,rect2.b.x)
	);
	
	int minY = min(
		min(rect1.a.y,rect2.a.y)
		, min(rect1.b.y,rect2.b.y)
	);
	
	struct rectangle surrounding = { {minX,minY}, {maxX,maxY} };
	
	return surrounding;
};

/**
 * main function for testing the smallestSurroundingRectangle function
 * @return the exit value of the program. 
 * 0 for correct behaviour.
 */
int main(void) {
	struct point a = { 2 , 2 };
	struct point b = { 4 , 4 };
	struct rectangle rect = { a, b };
	
	struct rectangle rect2 = { {3,3}, {6,9} };
	
	struct rectangle surr = smallestSurroundingRect(rect,rect2);
	
	printf("Smallest surrounding rectangle of [{%d, %d},{%d, %d}]" \
		"and [{%d, %d},{%d, %d}] is [{%d, %d},{%d, %d}]\n"
		, rect.a.x, rect.a.y, rect.b.x, rect.b.y
		, rect2.a.x, rect2.a.y, rect2.b.x, rect2.b.y
		, surr.a.x, surr.a.y, surr.b.x, surr.b.y
	); 		

	return 0; 		
}
